C
C     *****************                                                  
C     *               *                                               
C     * A L A M B D A *                                                   
C     *               *                                                 
C     *****************                                                  
C                                                                       
C     Program for calculation of WIMSD type lambda factor of X isotope                
C                                                                   
C
C     Francisco Leszczynski - November 1996 - CAB-CNEA
C             Last version:   March 1999
C
C     Input files: ABSXS (output of RMET21 calc with lambda option)
C                  RI.OUT (output of RMET21 calc with Ri.vs.XSb option
C                          for 10 XSb's)	
C    
C     Output file: ALAMBDA.OUT (main output)
C
C     Auxiliary generated file: XAVXB (inverted order of RI.OUT content)
C    
C-----------------------------------------------------------------------------
C              INPUT ON ABSXS (output of RMET21 with lambda calc option for X):
C
C                         ----------------------------------------------------
C                           Reg.1 (A80)   : Title
C
C                           Reg.2 (2I5)   : NGG number of energy groups
C                                           NX  number of cases 
C                                               (with diff.densities)
C
C                           Reg.3 (6E12.5)    : (EGG(I),I=1,NG) 
C                                               lethargy widths of groups 
C                                               (from high to low energy)
C
C                           Reg.4 (4E12.5):     PXU xs-pot reso isotope
C                                               PXH xs-pot H
C                                               PXX xs-pot X
C                                               ALU lambda of reso isotope
C
C                           Reg.5 (2E12.5):     DNH atomic density of H
C                                               DNX atomic density of X
C
C                          NGG values (I=1,NGG): 
C                          --------------------
C                           Reg.6I (E12.5): XSa-I output of RMET21 with isot.X)
C                         ----------------------------------------------------
C
      DIMENSION EGG(70),RUH(10),XSBB(10)
C
      CHARACTER*80 AAAA
   1  FORMAT(2E12.5)
   2  FORMAT(A4)
   3  FORMAT(E12.5)
   4  FORMAT(4E12.5)
   5  FORMAT(2E12.5)
   6  FORMAT(1X,'GROUP ',I3,' XSA = ',1PE12.5,' LAMBDA = ',1PE12.5)
   7  FORMAT(2I5)
   8  FORMAT(6E12.5)
   9  FORMAT(A80)
  10  FORMAT(' NO.OF GROUPS= ',I5,' NO.OF DENSITIES = ',I5/)
  11  FORMAT(' LETHARGY WIDTHS OF GROUPS:')
  12  FORMAT(/' XSpots.:Res.Isot.= ',E12.5,' - H= ',E12.5,' - X= ',E12.5
     1/' LAMBDA res.isot.= ',E12.5/)
  13  FORMAT(' ATOMIC DENS.: H= ',E12.5,' X= ',E12.5/)
  14  FORMAT(' RESULTS FOR EACH GROUP')
  15  FORMAT(/' AVERAGED RESULT (OVER THE GROUPS)')
  16  FORMAT(/' CASE = ',I5)
  17  FORMAT(4X,' XSA = ',1PE12.5,' LAMBDA = ',1PE12.5)
C
      OPEN(UNIT=5,FILE='ABSXS',FORM='FORMATTED',STATUS='OLD')
      OPEN(UNIT=6,FILE='ALAMBDA.OUT',FORM='FORMATTED',STATUS='NEW')
C
      ID10=10
      DNU=1.0
      DBXS1=0.005
      DAX=0.005
C
      READ(5,9)AAAA
      WRITE(6,9)AAAA
      READ(5,7)NGG,NX
      WRITE(6,10)NGG,NX
      WRITE(6,11)
      READ(5,8)(EGG(I),I=1,NGG)
      WRITE(6,8)(EGG(I),I=1,NGG)
      READ(5,4)PXU,PXH,PXX,ALU
      WRITE(6,12)PXU,PXH,PXX,ALU
C
      DO ICASE=1,NX
C
       WRITE(6,16)ICASE
       READ(5,5)DNH,DNX
       WRITE(6,13)DNH,DNX
       WRITE(6,14)
C
       IF(ICASE.EQ.1)CALL CONVER(NGG)
C
C
       DO J=1,10
        RUH(J)=0.0
       ENDDO
C
       DUGG=0.0
       RUHX=0.0
C
C      skip first extra group
C
       READ(5,3)AX
C
       DO IGG=2,NGG
C
        DUGG=DUGG+EGG(IGG)
        READ(5,3)AX
C 
        DBXS=DBXS1
        CALL LAMBDA(ID10,NTABXA,IGG,NGG,AX,DNU,DNH,DNX,PXU,PXH,PXX,
     1              BX,ALU,ALAM,DBXS,DAX,ITEDAX,MAIDBX,RUHX,
     2              EGG,RUH,XSBB)
        WRITE(6,6)IGG,AX,ALAM
C
       END DO
C
C  calcul of average lambda (over groups) 
C
       DO J=1,10
        RUH(J)=RUH(J)/DUGG
       ENDDO
       RUHX=RUHX/DUGG     
C
       DBXS=DBXS1
       IGG=1
       CALL LAMBDA1(ID10,NTABXA,IGG,NGG,RUHX,DNU,DNH,DNX,PXU,PXH,PXX,
     1              BX,ALU,ALAM,DBXS,DAX,ITEDAX,MAIDBX,RUHX,
     2              EGG,RUH,XSBB)
       WRITE(6,15)
       WRITE(6,17)RUHX,ALAM
       CLOSE(10)
C
      ENDDO
C
       CLOSE(5)
       CLOSE(6)
C
      STOP
      END         
C
C
C                                                                       
C     ***************                                                   
C     *             *                                                   
C     * L A M B D A *                                                   
C     *             *                                                   
C     ***************                                                   
C                                                                                                                                              
      SUBROUTINE LAMBDA(ID10,NTABXA,IGG,NGG,AX,DNU,DNH,DNX,PXU,PXH,PXX, 
     1                  BX,ALU,ALAM,DBXS,DAX,ITEDAX,MAIDBX,RUHX,
     2                  EGG,RUH,XSBB)
C                                                                       
      DIMENSION XSA(10),XSB(10),EGG(70),RUH(10),XSBB(10)
C
 2    FORMAT(5X,2E12.5)                                                 
C                                                                       
C     read XSB,XSA for interpolations                                
C                                                                     
      ITEDAX=0 
      ITEDBX=-1
      MAIDBX=0 
      NV=10                                                           
      OPEN(UNIT=10,FILE='XAVXB',FORM='FORMATTED',STATUS='OLD')
      DO I=1,10                                                      
       READ(ID10,2)XSB(I),XSA(I)                                     
       RUH(I)=RUH(I)+XSA(I)*EGG(IGG)
       XSBB(I)=XSB(I)
      END DO                                                         
C                                                                     
      DO I=1,10                                                       
       XSB1=XSB(I)                                                     
       XSB(I)=SQRT(XSB1)                                               
       ABS1=XSA(I)                                                     
       XSA(I)=ABS1/(1.0-(ABS1/XSB1))                                   
      END DO                                                          
C                                                                       
C     Obtention of mic XSA                                      
C                                                                       
      AX=AX/DNU                                                         
C                                                                       
C     Obtention of XSB corresponding to XSA                           
C                                                                       
      XAMIN=XSA(1)                                                      
      XAMAX=XSA(NV-1)                                                   
      IF(AX.LT.XAMIN)GO TO 40                                           
      IF(AX.GT.XAMAX)GO TO 50                                           
      XA=XSA(1)
      DO I=2,NV                                                      
       IF(AX.GE.XA)THEN
        II=I                                                              
        XA=XSA(I)                                                         
       ENDIF
      ENDDO
      IN1=II-1                                                          
      IN2=II                                                            
      IN3=II+1                                                          
      BXS=XSB(IN1)                                                      
      BXY=XSB(IN2)                                                      
      GO TO 60                                                          
 40   IN1=1                                                             
      IN2=2                                                             
      IN3=3                                                             
      DBXS=-DBXS                                                        
      BXS=XSB(IN1)                                                      
      BXY=5.0                                                           
      GO TO 60                                                          
 50   IN1=NV-2                                                          
      IN2=NV-1                                                          
      IN3=NV                                                            
      BXS=XSB(NV-1)                                                     
      BXY=BXS+100.0                                                     
 60   A1=XSA(IN1)                                                       
      A2=XSA(IN2)                                                       
      A3=XSA(IN3)                                                       
      B1S=XSB(IN1)                                                      
      B2S=XSB(IN2)                                                      
      B3S=XSB(IN3)                                                      
C                                                                       
C     XSb search                                                  
C                                                                       
      BXX=BXS                                                           
 70   DBXS=DBXS/10.
      ITEDBX=ITEDBX+1  
      IF(ITEDBX.GT.MAIDBX)MAIDBX=ITEDBX                                 
      BXS=BXX                                                           
 80   BXS=BXS+DBXS
      IF(BXY.EQ.5.0)GO TO 85                                            
      IF(BXS.GE.BXY)GO TO 70                                            
 85   IF(BXS.LE.0.0)GO TO 90
      A=BXS-B2S                                                         
      B=BXS-B3S                                                         
      C=B1S-B2S                                                         
      D=B1S-B3S                                                         
      E=BXS-B1S                                                         
      F=B2S-B3S                                                         
      G=B3S-B1S                                                         
      AX1=A*B*A1/(C*D)-E*B*A2/(C*F)-E*A*A3/(G*F)                        
      AXX=((AX1/AX)-1.0)*100.0                                          
      IF(AXX.LT.0.0)AXX=-AXX                                            
      IF(AXX.GE.DAX)ITEDAX=ITEDAX+1
      IF(AXX.GE.DAX)GO TO 80                                          
      BX=BXS*BXS                                                        
C                                                                       
C     Calcul of lambda                                                 
C                                                                       
  90  ALAM=(BX-ALU*PXU-(DNH/DNU)*PXH)*DNU/(DNX*PXX)                     
C
      RUHX=RUHX+AX*BX*EGG(IGG)/(AX+BX)
C                                                                       
      RETURN                                                            
      END                                                               
C                                                                     
C
C                                                                    
C     *****************                                                
C     *               *                                                 
C     * L A M B D A 1 *                                                
C     *               *                                               
C     *****************                                               
C                                                                      
      SUBROUTINE LAMBDA1(ID10,NTABXA,IGG,NGG,AX,DNU,DNH,DNX,PXU,PXH,PXX, 
     1                   BX,ALU,ALAM,DBXS,DAX,ITEDAX,MAIDBX,RUHX,
     2                   EGG,RUH,XSBB)
C                                                                      
      DIMENSION XSA(10),XSB(10),EGG(70),RUH(10),XSBB(10)
C                                                                   
      ITEDAX=0 
      ITEDBX=-1
      MAIDBX=0 
      NV=10                                                          
      DO I=1,10                                                    
       XSA(I)=RUH(I)
       XSB(I)=XSBB(I)
      END DO                                                          
C                                                                    
      DO I=1,10                                                      
      XSB1=XSB(I)                                                     
      XSB(I)=SQRT(XSB1)                                              
C      ABS1=XSA(I)                                                    
C      XSA(I)=ABS1/(1.0-(ABS1/XSB1))                                     
      END DO                                                          
C                                                                       
      AX=AX/DNU                                                        
C                                                                      
      XAMIN=XSA(1)                                                     
      XAMAX=XSA(NV-1)                                                  
      IF(AX.LT.XAMIN)GO TO 40                                          
      IF(AX.GT.XAMAX)GO TO 50                                          
      XA=XSA(1)
      DO I=2,NV                                                     
       IF(AX.GE.XA)THEN
        II=I                                                             
        XA=XSA(I)                                                        
       ENDIF
      ENDDO
      IN1=II-1                                                        
      IN2=II                                                            
      IN3=II+1                                                         
      BXS=XSB(IN1)                                                      
      BXY=XSB(IN2)                                                   
      GO TO 60                                                         
 40   IN1=1                                                             
      IN2=2                                                           
      IN3=3                                                            
      DBXS=-DBXS                                                        
      BXS=XSB(IN1)                                                      
      BXY=5.0                                                           
      GO TO 60                                                         
 50   IN1=NV-2                                                        
      IN2=NV-1                                                         
      IN3=NV                                                           
      BXS=XSB(NV-1)                                                    
      BXY=BXS+100.0                                                   
 60   A1=XSA(IN1)                                                      
      A2=XSA(IN2)                                                      
      A3=XSA(IN3)                                                       
      B1S=XSB(IN1)                                                     
      B2S=XSB(IN2)                                                      
      B3S=XSB(IN3)                                                     
C                                                                      
      BXX=BXS                                                         
 70   DBXS=DBXS/10.
      ITEDBX=ITEDBX+1  
      IF(ITEDBX.GT.MAIDBX)MAIDBX=ITEDBX                                
      BXS=BXX                                                          
 80   BXS=BXS+DBXS
      IF(BXY.EQ.5.0)GO TO 85                                           
      IF(BXS.GE.BXY)GO TO 70                                          
 85   IF(BXS.LE.0.0)GO TO 90
      A=BXS-B2S                                                         
      B=BXS-B3S                                                         
      C=B1S-B2S                                                        
      D=B1S-B3S                                                       
      E=BXS-B1S                                                        
      F=B2S-B3S                                                         
      G=B3S-B1S                                                        
      AX1=A*B*A1/(C*D)-E*B*A2/(C*F)-E*A*A3/(G*F)                        
      AXX=((AX1/AX)-1.0)*100.0                                         
      IF(AXX.LT.0.0)AXX=-AXX                                          
      IF(AXX.GE.DAX)ITEDAX=ITEDAX+1
      IF(AXX.GE.DAX)GO TO 80                                         
      BX=BXS*BXS                                                        
C                                                                       
  90  ALAM=(BX-ALU*PXU-(DNH/DNU)*PXH)*DNU/(DNX*PXX)                   
C                                                                     
      RETURN                                                            
      END                                                              
C
C                                                                       
      SUBROUTINE CONVER(NG)
C
C     *************   C O N V E R   ***************
C
C     OBTENTION OF FILE 'XAVXB' FROM FILE 'RI.OUT' (OUTPUT OF RMET21)
C  
C     Invert order of data (RI VS. XSB) as follow:
C
C               << RI.OUT>>                       << XAVXB >>
C        for each XSb all groups      ->    for each group all  XSb
C
C     Eliminates the first extra group from RI.OUT 
C   
      DIMENSION XSB(10,70),RIA(10,70)
C
  2   FORMAT(5X,2E12.5)
  3   FORMAT(I5,1P2E12.5)
C
      OPEN(UNIT=3,FILE='RI.OUT',FORM='FORMATTED',STATUS='OLD')
      OPEN(UNIT=7,FILE='XAVXB',FORM='FORMATTED',STATUS='NEW')
C
      DO J=1,10
       READ(3,2)XSBDU,RIADU 
       DO I=2,NG
        READ(3,2)XSB(J,I),RIA(J,I)
       END DO
      END DO
C
      DO I=2,NG
       DO J=1,10
        WRITE(7,3)I,XSB(J,I),RIA(J,I)
       END DO
      END DO
C
      CLOSE(3)
      CLOSE(7)
C
      RETURN
      END
